#!/usr/bin/env python3
# -*- coding: utf-8 -*-


__author__ = "Christian Heider Nielsen"
__doc__ = r"""
          This file is not for general use. Ode to python

           Created on 13/06/2020
           """

__all__ = []


def this_is_barely_legal():
    try:
        try:
            try:
                try:
                    try:
                        try:
                            try:
                                try:
                                    try:
                                        try:
                                            try:
                                                try:
                                                    try:
                                                        try:
                                                            try:
                                                                try:
                                                                    try:
                                                                        try:
                                                                            try:
                                                                                """
                                                                                try:
                                                                                  try:
                                                                                    try:
                                                                                      try:
                                                                                        try:
                                                                                          try:
                                                                                            try:
                                                                                              try:
                                                                                                try:
                                                                                                    ...
                                                                                                except:
                                                                                                    ...
                                                                                              except:
                                                                                                  ...
                                                                                            except:
                                                                                                ...
                                                                                          except:
                                                                                              ...
                                                                                        except:
                                                                                            ...
                                                                                      except:
                                                                                          ...
                                                                                    except:
                                                                                        ...
                                                                                  except:
                                                                                      ...
                                                                                except:
                                                                                  ...
                                                                                """
                                                                            except:
                                                                                ...
                                                                        except:
                                                                            ...
                                                                    except:
                                                                        ...
                                                                except:
                                                                    ...
                                                            except:
                                                                ...
                                                        except:
                                                            ...
                                                    except:
                                                        ...
                                                except:
                                                    ...
                                            except:
                                                ...
                                        except:
                                            ...
                                    except:
                                        ...
                                except:
                                    ...
                            except:
                                ...
                        except:
                            ...
                    except:
                        ...
                except:
                    ...
            except:
                ...
        except:
            ...
    except:
        ...


if __name__ == "__main__":
    # from __future__ import braces # TODO: Try disable comment

    # noinspection PyUnresolvedReferences
    import __hello__

    print()
    import this

    print()
    # noinspection PyUnresolvedReferences
    import antigravity

    print()
    print(this.i, this.d, this.c, this.s)
    print()

    # noinspection PyUnresolvedReferences
    # import that  # pip install that

    this_is_barely_legal()
